#!/usr/bin/env python
#####################################################################################
#
# Copyright (c) Members of the EGEE Collaboration. 2004.
# See http://eu-egee.org/partners/ for details on the copyright holders.
# For license conditions see the license file or http://eu-egee.org/license.html
#
#####################################################################################
#
# Template script for configuring the Condor service for gLite
# Alberto Di Meglio <alberto.di.meglio@cern.ch>
# Version info: $Id$
#
#####################################################################################

import os, re, sys, time
from gLiteInstallerLib import gLib

class Condor:

    def __init__(self,params):

        global glib

        # Initialize class attributes
        self.verbose = 0

        #initialize gLib
        self.glib = gLib()
        self.glib.set_params(params)

        # Check condor directory links (required by condor c)
        self.condor_location = params['condor.releasedir']
        if self.condor_location != "/opt/condor-c":
            if os.path.islink("/opt/condor-c"):
                os.remove("/opt/condor-c")
    
            os.symlink(self.condor_location, "/opt/condor-c")
            self.condor_location = "/opt/condor-c"

        # Initialize condor parameters
        self.condor_config = params['CONDOR_CONFIG']
        self.condor_config_local = "%s/local.%s/condor_config.local" % (self.condor_location, self.glib.hostname)
        if params.has_key('condor.blahgahp'):
            self.blahgahp =  params['condor.blahgahp']
        if params.has_key('condor.blahpollinterval'):
            self.blahpollinterval = params['condor.blahpollinterval']
        self.localdir = params['condor.localdir']
        self.daemonlist = params['condor.daemonlist']
        self.scheddinterval =  params['condor.scheddinterval']
        self.condoruser = params['condor.user']

        # Initialize Condor user uid and gid
        self.uid = 0
        self.gid = 0

        os.environ['CONDOR_CONFIG'] = self.condor_config
        if params.has_key('condor.version'):
            self.condor_version = params['condor.version']

    #####################################################################################
    # Start the Condor master as the condor user
    # Return: the Condor master process ID or 0 if it fails
    #####################################################################################

    def start(self):

        if self.uid == 0:
            print "Condor uid not defined. Run configure first"
            return 1

        condormaster = '%s/sbin/condor_master' % self.condor_location
        cmd = '/bin/su %s -c "export CONDOR_CONFIG=%s; %s"' % (self.condoruser, \
            self.condor_config, condormaster)
        if self.verbose:
            print "Executing command %s" % cmd
        os.system(cmd)

        # Check that the services have started
        cmpid = self.glib.getPID('condor_master')
        if cmpid != 0:
            print "The Condor master has been started with PID=%d" % cmpid
        else:
            print "Failed to start the Condor master"
            return 0

        pid = self.glib.getPID('condor_schedd')
        if pid != 0:
            print "The Condor schedd has been started with PID=%d" % pid
        else:
            print "Failed to start the Condor schedd"
            return 0

        pid = self.glib.getPID('condor_collector')
        if pid != 0:
            print "The Condor collector has been started with PID=%d" % pid
        else:
            print "Failed to start the Condor collector"
            return 0

        pid = self.glib.getPID('condor_negotiator')
        if pid != 0:
            print "The Condor negotiator has been started with PID=%d" % pid
        else:
            print "Failed to start the Condor negotiator"
            return 0

        return cmpid

    #####################################################################################
    # Stop the Condor master process
    #####################################################################################

    def stop(self):

        pid = self.glib.getPID('condor_master')
        if pid != 0:
            os.system("/usr/bin/killall -w condor_master")
            pid = self.glib.getPID('condor_master')
            if pid != 0:
                print "The Condor master was not terminated             ",
                self.glib.printErrorMessage()
            else:
                print "The Condor master has been terminated            ",
                self.glib.printOkMessage()
        else:
            if self.verbose:
                self.glib.printInfoMessage("The Condor master was not running")

        return 0

    #####################################################################################
    # Configure the Condor-C system for a gLite CE
    #####################################################################################

    def configureCondorC(self):

        self.glib.verbose = self.verbose

        # Check condor directory
        if not os.path.exists(self.condor_location):
            print "The Condor package doesn't seem to be installed in %s. Configuration cannot continue" % self.condor_location
            return 1

        # Create the condor user
        (self.uid,self.gid) = self.glib.add_user(self.condoruser)

        # (Re)generate Condor config files (this is the scriptlet in the condor rpm)
        os.system("export PATH=/usr/bin:/bin:$PATH ; cd %s ; touch ./ignore.me ; tar cf ./release.tar ignore.me ; rm ./ignore.me ; mv sbin sbin.tmp ; ./condor_configure --install --owner=%s ; mv sbin.tmp sbin ; rm ./ignore.me ; rm ./release.tar ; cd -" \
            % (self.condor_location, self.condoruser))

        condor_local_config = '%s/local.%s/condor_config.local' % (self.condor_location, self.glib.hostname)
        os.system("cp %s %s.1" % (condor_local_config,condor_local_config))
        os.system("cp %s %s.1" % (os.environ['CONDOR_CONFIG'],os.environ['CONDOR_CONFIG']))

        self.setValue("RELEASE_DIR", self.condor_location)
        self.setValue("LOCAL_DIR", self.localdir)
        self.setValue("SCHEDD_INTERVAL", self.scheddinterval)
        self.setValue("SCHEDD_INTERVAL", self.scheddinterval)
        self.setValue("GRIDMANAGER_CHECKPROXY_INTERVAL", "60")
        self.setValue("GRIDMANAGER_CHECKPROXY_INTERVAL", "60")
        self.setValue("GRIDMANAGER_MAX_PENDING_REQUESTS", "15")
        self.setValue("BLAH_GAHP", self.blahgahp)
        self.setValue("INFN_JOB_POLL_INTERVAL", self.blahpollinterval)
        self.setValue("LOCK", "$(LOG)")
        self.setValue("DAEMON_LIST", self.daemonlist)
        self.setValue("CONDOR_IDS", "%s.%s" % (self.uid,self.gid))
        self.setValue("JAVA_HOME", os.environ['JAVA_HOME'])
        self.setValue("QUEUE_SUPER_USERS", "")
        self.setValue("SEC_DEFAULT_NEGOTIATION", "REQUIRED")
        self.setValue("SEC_DEFAULT_AUTHENTICATION", "REQUIRED")
        self.setValue("SEC_DEFAULT_AUTHENTICATION_METHODS", "FS,GSI")
        self.setValue("GSI_DAEMON_CERT", "$ENV(X509_DUPLICATE_USER_PROXY)")
        self.setValue("GSI_DAEMON_KEY", "$ENV(X509_DUPLICATE_USER_PROXY)")
        if self.glib.params.has_key('condor.HIGHPORT'):
            self.setValue("HIGHPORT", self.glib.params['condor.HIGHPORT'])
        if self.glib.params.has_key('condor.LOWPORT'):
            self.setValue("LOWPORT", self.glib.params['condor.LOWPORT'])


        # Set permissions on condor local dir
        self.condorDirs()

        # Copy local config file to condor local dir
        os.system("cp %s %s/condor_config.local" % (self.condor_config_local, self.localdir))

        return 0

    #####################################################################################
    # Restart the Condor master process
    #####################################################################################

    def restart(self):
        self.stop()
        pid = self.start()

        return pid

    #####################################################################################
    # Create and set permissions for default condor directories
    #####################################################################################

    def condorDirs(self):
        if self.verbose:
            print "Verifying standard Condor directories..."

        (self.uid,self.gid) = self.glib.get_user_info(self.condoruser)
        os.chown(self.condor_config_local,self.uid,self.gid)

        if not os.path.exists(self.localdir):
            os.makedirs(self.localdir)
        if not os.path.exists("%s/log" % self.localdir):
            os.mkdir("%s/log" % self.localdir)
        if not os.path.exists("%s/spool" % self.localdir):
            os.mkdir("%s/spool" % self.localdir)
        if not os.path.exists("%s/execute" % self.localdir):
            os.mkdir("%s/execute" % self.localdir)

        os.system("/bin/chown -R %s:%s %s" % (self.uid,self.gid,self.localdir))

        os.system("/bin/rm -rf -R /tmp/condor*")

        print "Standard Condor directories created/verified in %s" % self.localdir


    #######################################################################################
    #
    #                MANAGEMENT OF CONDOR PROCESSES
    #
    #######################################################################################

    ########################################################################################
    # Get processes running in condor
    ########################################################################################
    def ps(self):
        psList = os.popen('condor_q','r').readlines()
        if len(psList) > 3:
            return os.popen('condor_q','r').readlines()[4:]
        else:
            return []
        
    def kill(self,pid):
        os.system('condor_rm %s' % pid)
    
    def killName(self, name):
        count = 0
        for process in self.ps():
            items = process.split()
            if len(items) == 9:
                if (items[8].strip() == name):
                    self.kill(items[0].strip())
                    count = count + 1

        print("Killed %d %s processes)" %(count, name))

    ########################################################################################
    # Get condor pid of the process 'name'
    # returns:
    #        >0 : Process pid
    #        0 : No process called 'name' is running
    #        <0 : Multiple processes of this name running. Returned -number_of_processes
    ########################################################################################
    def getPid(self, name):
        count = 0
        for process in self.ps():
            if (process.split()[8].strip() == name):
                count = count + 1
                pid = process.split()[0].strip()
        if not (count == 1):
            pid = -count
        return pid
        
    ########################################################################################
    # Set/update condor config parameter
    # returns:
    #        0 : Value set successfully
    #        1 : Value not set successfully
    ########################################################################################
    def setValue(self, param, value):

        # First check if value is already set in the main config file and change it there
        try:
            fn = open(self.condor_config,'r')
        except:
            self.glib.printErrorMessage("%s could not be opened for writing." % self.condor_config)
            return 1

        ft = open("/tmp/condor_config.tmp",'w')
        for l in fn:
            if l.strip().lstrip("#").startswith(param):
                ft.write("%s = %s\n" % (param, value) )
            else:
                ft.write(l)

        fn.close()
        ft.close()

        os.system("mv /tmp/condor_config.tmp %s" % self.condor_config)
        
        # Then check if value is already set in the local config file and change it there
        # or add it
        try:
            fn = open(self.condor_config_local,'r')
        except:
            self.glib.printErrorMessage("%s could not be opened for writing." % self.condor_config_local)
            return 1

        valueFound = 0
        ft = open("/tmp/condor_config_local.tmp",'w')
        for l in fn:
            if l.strip().startswith(param):
                ft.write("%s = %s\n" % (param, value) )
                valueFound = 1
            else:
                ft.write(l)

        fn.close()
        ft.close()

        os.system("mv /tmp/condor_config_local.tmp %s" % self.condor_config_local)

        if not valueFound:
            fn = open(self.condor_config_local,'a')
            fn.write("\n# New parameter added by gLite configurator\n")
            fn.write("%s = %s\n\n" % (param, value) )
            fn.close
            
        return 0
