#!/usr/bin/env python
#####################################################################################
#
# Copyright (c) Members of the EGEE Collaboration. 2004.
# See http://eu-egee.org/partners/ for details on the copyright holders.
# For license conditions see the license file or http://eu-egee.org/license.html
#
#####################################################################################
# gLiteCheckWebServiceStatus
#
# Script to check status of a webservice
#
# Joachim Flammer <Joachim.Flammer@cern.ch>
#
# Version info: $Id: gLiteCheckWebserviceStatus.py,v 1.1.2.1 2005/12/06 14:30:44 flammer Exp $
#
# Usage: python gLiteCheckWebserviceStatus [-h|-v|--help|--version|--verbose|--endpoint|--certificate|--key]
#        -h, --help      print usage info
#        -v, --version   show version
#
#        --verbose               verbosity on
#        --endpoint=URL          endpoint to be checked
#        --certificate=certFile  certificate file (default: )
#        --key=keyFile           keyFile (default: )
#        --host=hostName         hostname (default: localhost)
#        --port=port             port (default: 8443)
#        --endpoint=URL          endpoint (default: '')
#        --certificate=certFile  certificate file (default: '/etc/grid-security/hostcert.pem')
#        --key=keyFile           keyFile (default: '/etc/grid-security/hostkey.pem')"
#
#        --successMessage='message'  message to be printed in case of success"
#        --failureMessage='message'  message to be printed in case of failure"
#
# Return codes: 0 - service running
#               1 - service not running
#
#####################################################################################

import os
import string
import socket
import sys
import getopt
import commands
import httplib

from gLiteInstallerLib import gLib

# Set global variables here 
global verbose

class gLiteCheckWebserviceStatus:

    #################################################################
    # init 
    #
    # init service class
    #
    #################################################################
    def __init__(self):
        
        self.verbose = 0
        self.version = "1.0.0"
        self.name    = "gliteCheckWebserviceStatus"
        
        # default values
        self.endpoint        = ""
        self.certificateFile = "/etc/grid-security/hostcert.pem"
        self.keyFile         = "/etc/grid-security/hostkey.pem"
        self.port            = "8443"
        self.hostName        = "localhost"
        self.failureMessage  = "Service is not running"
        self.successMessage  = "Service is running"
        
    
    #################################################################
    # copyright 
    #
    # show copyright
    #
    #################################################################
    def copyright(self):

        print '\nCopyright (c) Members of the EGEE Collaboration. 2004'
        print 'See http://eu-egee.org/partners/ for details on the copyright holders'
        print 'For license conditions see the license file or http://eu-egee.org/license.html'


    #################################################################
    # showVersion 
    #
    # show version
    #
    #################################################################
    def showVersion(self):

        print '\n%s  v. %s\n' % (self.name,self.version)
    

    #################################################################
    # usage 
    #
    # print usage
    #
    #################################################################
    def usage(self,msg = ""):

        if msg:
            print "\n%s" % (msg)
        
        self.copyright()
        self.showVersion()
    
        print "Script to check status of a webservice by using the url"
        print "    http(s)://HOSTNAME:PORT/ENDPOINT"
        print
        print "Usage: python gLiteCheckWebserviceStatus [-h|-v|--help|--version|--verbose|--endpoint|--certificate|--key]"
        print "        -h, --help                print usage info"
        print "        -v, --version             show version"
        print "        --verbose                 verbosity on"
        print
        print "        --host        hostName    hostname (default: localhost)"
        print "        --port        portNumber  port (default: 8443)"
        print "        --endpoint    URL         endpoint (default: '')"
        print "        --certificate certFile    certificate file (default: '/etc/grid-security/hostcert.pem')"
        print "        --key         keyFile     key file (default: '/etc/grid-security/hostkey.pem')"
        print
        print "        --successMessage 'message'  message to be printed in case of success"
        print "        --failureMessage 'message'  message to be printed in case of failure"
        print 
        print "Return codes: 0 - service running"
        print "              1 - service not running"
    

    #################################################################
    # getStatus 
    #
    # check status of service
    #
    # return status, code, description
    #        where status = 0: ok, 
    #              status = 1: failed
    #              code    return code from https request (200=ok,111=connection refused, 404=file not found ....)
    #              message message from https request
    #
    #################################################################
    def getStatus(self):
        
        if self.verbose:
            if self.port=="8080":
                print "Checking: http://%s:%s/%s" % (self.hostName, self.port, self.endpoint)
            else:
                print "Checking: https://%s:%s/%s" % (self.hostName, self.port, self.endpoint)
                
            print "  with key file:         %s" % self.keyFile
            print "  with certificate file: %s" % self.certificateFile
            
        # check if certificate exists:
        if not os.path.exists(self.keyFile):
            return (1,111,"ERROR: key file does not exist!")
        
        if not os.path.exists(self.certificateFile):
            return (1,111,"ERROR: certificate file does not exist!")
        
        try:
            connection = httplib.HTTPSConnection(self.hostName,string.atoi(self.port),self.keyFile,self.certificateFile)
            connection.request("GET", self.endpoint)
            response = connection.getresponse()
        except socket.error, msg:
            return (1,msg[0],msg[1])

        # check if response was ok (200)    
        if response.status==200:
            return (0, response.status,response.reason)
        else:
            if response.status==404:
                return (1, response.status, "Endpoint '%s' not found." % response.reason)
            else:
                return (1,response.status,response.reason)

                        
#-------------------------------------------------------------------------------
#  Main program begins here 
#-------------------------------------------------------------------------------

if __name__ == '__main__':

    # ------------------------------------------------------------          
    # Instantiate the service classes
    # ------------------------------------------------------------
    service = gLiteCheckWebserviceStatus()
                
    # ------------------------------------------------------------
    # Check command line
    # ------------------------------------------------------------       
    try:   
        opts, args = getopt.getopt(sys.argv[1:],  'h:v', ['help', 'version', 'verbose', 'port=', 
                                    'host=', 'endpoint=', 'certificate=', 'key=', 
                                    'failureMessage=', 'successMessage='])                                            
    except getopt.GetoptError:          
        service.usage()                         
        sys.exit(2)        
    
    for o, a in opts:
           
        # help
        if o in ("-h", "--help"):
            service.usage()
            sys.exit(0)
        
        # version            
        elif o in ("-v", "--version"):
            service.showVersion()
            sys.exit(0)

        elif o == "--verbose":
            service.verbose = 1                
            
        elif o == "--port":
            service.port = args
        
        elif o == "--host":
            service.hostName = a
        
        elif o == "--endpoint":
            service.endpoint = a
        
        elif o == "--certificate":
            service.certificateFile = a
        
        elif o == "--key":
            service.keyFile = a
        
        elif o == "--successMessage":
            service.successMessage = a
        
        elif o == "--failureMessage":
            service.failureMessage = a            

    # ------------------------------------------------------------
    # Check status
    # ------------------------------------------------------------
    status = service.getStatus()


    if status[0] == 0:
        # -----------------------------------------------------------
        # running
        # -----------------------------------------------------------

        if service.verbose:
            print status[2]

        print service.successMessage            
        sys.exit(0)
        
    else:
        if status[1]==404:
            
            # -----------------------------------------------------------
            # failed
            # -----------------------------------------------------------
            
            if service.verbose:
                print "Endpoint '%s' not found." % status[2]
        
            print service.failureMessage                
            sys.exit(1)
            
        else:

            # -----------------------------------------------------------
            # failed
            # -----------------------------------------------------------

            if service.verbose:
                print status[2]

            print service.failureMessage
            sys.exit(1)                    
